Pro FITS_Read_TOD, file, arch, NoHK = NoHK
;+
; NAME:
;	FITS_READ_TOD
; PURPOSE:
;	Reads a MAP time-ordered data (TOD) archive file that was written in
;	FITS format.
; CALLING SEQUENCE:
;	FITS_READ_TOD, file, arch, [/NOHK] 
; INPUTS:
;	file - The name of the FITS file to read.
; OUTPUTS:
;	arch - The TOD data structure.    See the function TOD_FORMAT()
;              for additional information about this structure
; OPTIONAL INPUT KEYWORD:
;       /NOHK - if set, then no housekeeping information is returned, and 
;                only the time stamp and science data are defined in the 
;                output structure
; EXAMPLE:
;       Read the time-ordered data for day 221 of year 2002 into an output
;       structure
;       IDL>  file ='MAP_tod_20022202357_20022212357.fits'
;       IDL>  FITS_READ_TOD, file, arch
;    
; PROCEDURES USED:
;       tod_format(), jul2ts                            ;MAP Library
;       fits_open,fits_read,fits_close, tbinfo,tbget()    IDLAstro Library
; MODIFICATION HISTORY:
;	Written by Michael R. Greason, SSAI, 27 September 2002.
;       Additional Vectorization, NoHK keyword W. Landsman SSAI 21 November 2002
;-
On_error, 2
;
; Check arguments.
;
 If (N_params() LT 2) Then begin 
      print, 'Syntax - FITS_READ_TOD, file, arch, [/NoHK]'
      return
 endif
;
; Open the FITS file and find the number of extensions.
;
 fits_open, file, fcb            ;Extract the file control block
 If (fcb.nextend LT 5) Then message, 'Invalid MAP TOD FITS file!'
;
; Extract the number of records in the file and allocate space.
;
 narch = sxpar(fcb.hmain, 'NUMREC')
 If (narch LE 0) Then message, 'Invalid FITS file -- No data'
;
 arch = TOD_Format(narch, NoHK = nohk)         ;Return empty output structure
;
; Extension 1:  Fill the array from the Meta data table.
;
 if not keyword_set(nohk) then begin
 arch.calflag = 1
 fits_read,fcb,tab,eh,extname = 'Meta Data Table',message=msg,/No_Abort
 If (msg NE '') Then message, 'Invalid FITS file -- Meta data'

If (size(tab,/dimen))[1] GT narch Then tab = tab[*,0:narch-1]
tbinfo,eh,tb_str
;
 s = size(arch[0].quaternions,/dimen)
 arch.Quaternions = reform(tbget(tb_str,tab,'QUATERN'), s[0],s[1], narch)
;
 arch.Position    = tbget(tb_str,tab,'Position') 
 arch.Velocity    = tbget(tb_str,tab,'Velocity') 
 arch.GeoPosition = tbget(tb_str,tab, 'GeoPos') 
 arch.GeoVelocity = tbget(tb_str,tab, 'GeoVel') 
;
 arch.nsci        =  tbget(tb_str,tab,'NSCI')
 arch.ndeu        =  tbget(tb_str,tab,'NDIHK')
 endif
;
; Extension 2: Fill the array from the science data table.
;
 fits_read,fcb,tab,eh,extname = 'Science Data Table',message=msg,/No_Abort

 If (msg NE '') Then message, 'Invalid FITS file -- Science'
;
 tbinfo,eh,tb_str
 tstab = Jul2TS(tbget(tb_str,tab, 'Time'))
   dimen = size(arch.sci,/dimen)
    arch.sci.Day  = reform(tstab[0,*], dimen)
    arch.sci.Time = reform(tstab[1,*], dimen)
    arch.sci.GenFlags = reform( tbget(tb_str,tab,'GenFlags'), dimen)
    dimen = size(arch.sci.daflags,/dimen)
    arch.sci.DAFlags  = reform ( tbget(tb_str,tab,'DAFlags'), dimen)
    if not keyword_set(nohk) then begin
    dimen = size(arch.sci.error1,/dimen)
    arch.sci.Error1   = reform( tbget(tb_str,tab,'Error1'), dimen)
    arch.sci.Error2   = reform( tbget(tb_str,tab,'Error2'), dimen)
    endif

    dimen = size(arch.sci.k,/dimen)
    k1 = reform( tbget(tb_str,tab,'K1'),dimen[1:3] )
    t = make_array(dimen=[dimen[1:3],dimen[0]],/float,/nozero)
    for i=0,3 do t[0,0,0,i] = k1
    arch.sci.k = transpose(t,[3,0,1,2])

    ka1 = reform( tbget(tb_str,tab,'KA1'),dimen[1:3] )
    t = make_array(dimen=[dimen[1:3],dimen[0]],/float,/nozero)
    for i=0,3 do t[0,0,0,i] = ka1
    arch.sci.ka = transpose(t,[3,0,1,2])
  
    dimen = size(arch.sci.q,/dimen)
    q1 = reform( tbget(tb_str,tab,'Q1'),dimen[1:3] )
    q2 = reform( tbget(tb_str,tab,'Q2'),dimen[1:3] )
    t = make_array(dimen=[dimen[1:3],dimen[0]],/float,/nozero)
    for i=0,3 do t[0,0,0,i] = q1
    for i=4,7 do t[0,0,0,i] = q2
    arch.sci.q = transpose(t,[3,0,1,2])

    dimen = size(arch.sci.v,/dimen)
    v1 = reform( tbget(tb_str,tab,'V1'),dimen[1:3] )
    v2 = reform( tbget(tb_str,tab,'V2'),dimen[1:3] )
    t = make_array(dimen=[dimen[1:3],dimen[0]],/float,/nozero)
    for i=0,3 do t[0,0,0,i] = v1
    for i=4,7 do t[0,0,0,i] = v2
    arch.sci.v = transpose(t,[3,0,1,2])

    dimen = size(arch.sci.w,/dimen)
    w1 = reform( tbget(tb_str,tab,'W1'),dimen[1:3] )
    w2 = reform( tbget(tb_str,tab,'W2'),dimen[1:3] )
    w3 = reform( tbget(tb_str,tab,'W3'),dimen[1:3] )
    w4 = reform( tbget(tb_str,tab,'W4'),dimen[1:3] )
     t = make_array(dimen=[dimen[1:3],dimen[0]],/float,/nozero)
    for i=0,3 do t[0,0,0,i] = w1
    for i=4,7 do t[0,0,0,i] = w2
    for i=8,11 do t[0,0,0,i] = w3
    for i=12,15 do t[0,0,0,i] = w4
    arch.sci.w = transpose(t,[3,0,1,2])
;
; Extension 3: Fill the array from the Analog Instrument HouseKeeping (AIHK) 
; data table.
;
 if not keyword_set(Nohk) then begin
 fits_read,fcb,tab,eh,extname = 'AIHK Data Table',message=msg,/No_Abort
 If (msg NE '') Then message, 'Invalid FITS file -- AIHK Data Table'
;
 tbinfo,eh,tb_str
 sweep = tbget(tb_str,tab,'Sweep')
 pdu = tbget(tb_str,tab,'PDU')
 aeu1 = tbget(tb_str,tab,'AEU1')
 aeu2 = tbget(tb_str,tab,'AEU2')
 awin1 = tbget(tb_str,tab,'AWIN1')
 awin2 = tbget(tb_str,tab,'AWIN2')
 counters = tbget(tb_str, tab, 'COUNTERS')

 tstab = Jul2TS(tbget(tb_str,tab,'Time'))
 g = where(sweep EQ 1, Ng)
 if Ng GT 0 then begin
    arch.aeu.Day = reform(tstab[0,g])
    arch.aeu.Time = reform(tstab[1,g])
    arch.aeu.SWEEP1.PDU_Analog = pdu[*,g]
    arch.aeu.Sweep1.AEU_Analog1 = aeu1[*,g] 
    arch.aeu.Sweep1.AEU_Analog2 = aeu2[*,g]
    arch.aeu.Sweep1.AEU_Window1 = awin1[*,g]
    arch.aeu.Sweep1.AEU_Window2 = awin2[*,g]
    arch.aeu.Counters  	 = counters[*,g]
 endif

 g = where(sweep NE 1, Ng)
 if Ng GT 0 then begin 
      arch.aeu.Sweep2.PDU_Analog  = pdu[*,g]
      arch.aeu.Sweep2.AEU_Analog1 = aeu1[*,g]
      arch.aeu.Sweep2.AEU_Analog2 = aeu2[*,g]
      arch.aeu.Sweep2.AEU_Window1 = awin1[*,g]
      arch.aeu.Sweep2.AEU_Window2 = awin2[*,g]
 endif 
 endif
;
; Fill the array from the Digital Instrument HouseKeeping (DIHK) data table.
;
 if not keyword_set(NoHK) then begin
 fits_read,fcb,tab,eh,extname = 'DIHK Data Table',message=msg,/No_Abort
 If (msg NE '') Then message, 'Invalid FITS file -- DIHK Data Table'
 ;
 tbinfo,eh,tb_str
 f_ind = tbget(tb_str,tab,'FrmInd') - 1
 d_ind = tbget(tb_str,tab,'DihkInd') - 1
 data = tbget(tb_str,tab,'Data')
 N = sxpar(eh, 'NAXIS2') - 1
 tstab = Jul2TS(tbget(tb_str,tab,'Time'))
  
 For i = 0, N Do Begin
     arch[f_ind[i]].deu[d_ind[i]].Day  = tstab[0,i]
     arch[f_ind[i]].deu[d_ind[i]].Time = tstab[1,i]
     arch[f_ind[i]].deu[d_ind[i]].Data = data[*,i]
 EndFor
 endif

 fits_close,fcb

 Return
 End
